Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 82568)
+++ channels/chan_sip.c	(working copy)
@@ -185,6 +185,7 @@
 static int max_expiry = DEFAULT_MAX_EXPIRY;        /*!< Maximum accepted registration time */
 static int default_expiry = DEFAULT_DEFAULT_EXPIRY;
 static int expiry = DEFAULT_EXPIRY;
+static int remotemode = 0;
 
 #ifndef MAX
 #define MAX(a,b) ((a) > (b) ? (a) : (b))
@@ -2152,6 +2153,8 @@
 static void ast_sip_ouraddrfor(struct in_addr *them, struct sockaddr_in *us)
 {
 	struct sockaddr_in theirs;
+	/* Always reset 'remotemode' to 0 */
+	remotemode = 0;
 	/* Set want_remap to non-zero if we want to remap 'us' to an externally
 	 * reachable IP address and port. This is done if:
 	 * 1. we have a localaddr list (containing 'internal' addresses marked
@@ -2189,6 +2192,7 @@
 			*us = externip;
 		else
 			ast_log(LOG_WARNING, "stun failed\n");
+		remotemode = 1;
 		ast_debug(1, "Target address %s is not local, substituting externip\n", 
 			ast_inet_ntoa(*(struct in_addr *)&them->s_addr));
 	} else if (bindaddr.sin_addr.s_addr) {
@@ -5011,7 +5015,10 @@
 	p->ocseq = INITIAL_CSEQ;
 
 	if (sip_methods[intended_method].need_rtp) {
-		p->rtp = ast_rtp_new_with_bindaddr(sched, io, 1, 0, bindaddr.sin_addr);
+		if (remotemode)
+			p->rtp = ast_remote_rtp_new_with_bindaddr(sched, io, 1, 0, bindaddr.sin_addr);
+		else
+			p->rtp = ast_rtp_new_with_bindaddr(sched, io, 1, 0, bindaddr.sin_addr);
 		/* If the global videosupport flag is on, we always create a RTP interface for video */
 		if (ast_test_flag(&p->flags[1], SIP_PAGE2_VIDEOSUPPORT))
 			p->vrtp = ast_rtp_new_with_bindaddr(sched, io, 1, 0, bindaddr.sin_addr);
@@ -12335,6 +12342,12 @@
 
 		/* Recalculate our side, and recalculate Call ID */
 		ast_sip_ouraddrfor(&p->sa.sin_addr, &p->ourip);
+		/* Now that we really know if the destination is remote, we can
+		 * construct a proper Remote  RTP header if we need to. */
+		if (p->rtp && remotemode) {
+			ast_rtp_destroy(p->rtp);
+			p->rtp = ast_remote_rtp_new_with_binaddr(sched, io, 1, 0, bindaddr.sin_addr);
+		}
 		build_via(p);
 		build_callid_pvt(p);
 		ast_cli(fd, "Sending NOTIFY of type '%s' to '%s'\n", argv[2], argv[i]);
